"use strict";
/**
 * @athenna/core
 *
 * (c) João Lenon <lenon@athenna.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = void 0;
const path_1 = require("path");
const ioc_1 = require("@athenna/ioc");
const utils_1 = require("@secjs/utils");
const Logger_1 = require("./Utils/Logger");
const ResolveClassExport_1 = require("./Utils/ResolveClassExport");
const AthennaErrorHandler_1 = require("./Utils/AthennaErrorHandler");
const NotBootedException_1 = require("./Exceptions/NotBootedException");
const AlreadyBootedException_1 = require("./Exceptions/AlreadyBootedException");
const AlreadyShutdownException_1 = require("./Exceptions/AlreadyShutdownException");
class Application {
    constructor(extension) {
        this.container = new ioc_1.Ioc();
        this.httpServer = null;
        this.httpRoute = null;
        this.extension = extension;
    }
    /**
     * An instance of the Ioc class that is a Monostate with
     * the Awilix container inside.
     *
     * @return Ioc
     */
    getContainer() {
        return this.container;
    }
    /**
     * An instance of the Http class or an exception if it's not
     * booted.
     *
     * @return Http
     */
    getHttpServer() {
        if (!this.httpServer) {
            throw new NotBootedException_1.NotBootedException('HttpServer');
        }
        return this.httpServer;
    }
    /**
     * An instance of the Router class or an exception if it's not
     * booted.
     *
     * @return HttpRoute
     */
    getHttpRoute() {
        if (!this.httpRoute) {
            throw new NotBootedException_1.NotBootedException('HttpServer');
        }
        return this.httpRoute;
    }
    // TODO
    // async bootWorker() {}
    // TODO
    // async shutdownWorker() {}
    // TODO
    // async bootConsole() {}
    // TODO
    // async shutdownConsole() {}
    /**
     * Boot a new HttpServer inside this Application instance.
     *
     * @return void
     */
    async bootHttpServer() {
        if (this.httpServer) {
            throw new AlreadyBootedException_1.AlreadyBootedException('HttpServer');
        }
        this.httpServer = this.container.safeUse('Athenna/Core/HttpServer');
        this.httpRoute = this.container.safeUse('Athenna/Core/HttpRoute');
        /**
         * Resolve the Kernel file inside Http directory. It's
         * extremely important to call this method before preloading
         * the routes/http file because of named middlewares
         */
        this.resolveHttpKernel();
        /**
         * Preload default http route file
         */
        this.preloadFile(utils_1.Path.pwd('routes/http'));
        this.httpServer.setErrorHandler(AthennaErrorHandler_1.AthennaErrorHandler.http);
        this.httpRoute.register();
        const port = Config.get('http.port');
        const host = Config.get('http.host');
        await this.httpServer.listen(port, host);
        Logger_1.Logger.log(`Http server started on http://${host}:${port}`);
        return this.httpServer;
    }
    /**
     * Shutdown the HttpServer inside this Application instance.
     *
     * @return void
     */
    async shutdownHttpServer() {
        if (!this.httpServer) {
            throw new AlreadyShutdownException_1.AlreadyShutdownException('HttpServer');
        }
        await this.httpServer.close();
        this.httpRoute = null;
        this.httpServer = null;
    }
    /**
     * Preload the file according to filePath. This is usefully
     * to preload default files of each type of application. Such
     * as routes and Kernels
     *
     * @param filePath
     * @private
     */
    preloadFile(filePath) {
        const { dir, name } = path_1.parse(filePath);
        Logger_1.Logger.log(`Preloading ${name} file`);
        require(`${dir}/${name}${this.extension}`);
    }
    /**
     * Resolve the Kernel of the http server
     *
     * @private
     */
    resolveHttpKernel() {
        const { dir, name } = path_1.parse(utils_1.Path.app('Http/Kernel'));
        const HttpKernel = ResolveClassExport_1.ResolveClassExport.resolve(require(`${dir}/${name}${this.extension}`));
        const httpKernel = new HttpKernel();
        const container = this.getContainer();
        const httpServer = this.getHttpServer();
        /**
         * Binding the named middlewares inside the container and
         * creating a simple alias for it.
         */
        Object.keys(httpKernel.namedMiddlewares).forEach(key => {
            const Middleware = ResolveClassExport_1.ResolveClassExport.resolve(httpKernel.namedMiddlewares[key]);
            if (!container.hasDependency(`App/Middlewares/${Middleware.name}`)) {
                container.bind(`App/Middlewares/${Middleware.name}`, Middleware);
            }
            container.alias(`App/Middlewares/Names/${key}`, `App/Middlewares/${Middleware.name}`);
        });
        /**
         * Resolving global middlewares inside the Http server.
         */
        httpKernel.globalMiddlewares.forEach(globalMiddleware => {
            globalMiddleware = ResolveClassExport_1.ResolveClassExport.resolve(globalMiddleware);
            if (globalMiddleware.handle) {
                httpServer.use(globalMiddleware.handle, 'handle');
            }
            if (globalMiddleware.intercept) {
                httpServer.use(globalMiddleware.intercept, 'intercept');
            }
            if (globalMiddleware.terminate) {
                httpServer.use(globalMiddleware.terminate, 'terminate');
            }
        });
    }
}
exports.Application = Application;
